/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.user;

import java.util.logging.Level;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.operator.Config;
import de.ozgcloud.operator.keycloak.OzgCustomResourceStatus;
import io.javaoperatorsdk.operator.api.reconciler.Cleaner;
import io.javaoperatorsdk.operator.api.reconciler.Context;
import io.javaoperatorsdk.operator.api.reconciler.ControllerConfiguration;
import io.javaoperatorsdk.operator.api.reconciler.DeleteControl;
import io.javaoperatorsdk.operator.api.reconciler.Reconciler;
import io.javaoperatorsdk.operator.api.reconciler.UpdateControl;
import lombok.extern.java.Log;

@Log
@ControllerConfiguration
@Component
public class KeycloakUserReconciler implements Reconciler<OzgKeycloakUser>, Cleaner<OzgKeycloakUser> {

	@Autowired
	private KeycloakUserService service;
	@Autowired
	private KeycloakUserPreconditionService preconditionService;

	@Override
	public UpdateControl<OzgKeycloakUser> reconcile(OzgKeycloakUser resource, Context<OzgKeycloakUser> context) {
		var userName = resource.getMetadata().getName();
		var namespace = resource.getMetadata().getNamespace();

		log.info(String.format("Reconciling user %s...", userName));
		log.info("User reconciler reconcile, keep after delete is set to: " + resource.getSpec().isKeepAfterDelete());

		try {
			var preconditionError = preconditionService.getPreconditionErrors(resource);
			if (preconditionError.isPresent()) {
				var errorMessage = preconditionError.get();

				log.warning(String.format("Could not reconcile user %s in namespace %s: %s", userName, namespace, errorMessage));

				return UserUpdateControlBuilder.fromResource(resource)
						.withStatus(OzgCustomResourceStatus.IN_PROGRESS)
						.withMessage(errorMessage)
						.withReschedule(Config.RECONCILER_RETRY_SECONDS)
						.build();
			}

			service.createOrUpdateUser(resource.getSpec(), namespace);

			return UserUpdateControlBuilder.fromResource(resource).withStatus(OzgCustomResourceStatus.OK).build();

		} catch (Exception e) {
			var errorMessage = e.getMessage();

			log.log(Level.SEVERE, String.format("Could not reconcile user %s for namespace %s: %s", userName, namespace, errorMessage), e);

			return UserUpdateControlBuilder.fromResource(resource)
					.withStatus(OzgCustomResourceStatus.ERROR)
					.withMessage(errorMessage)
					.withReschedule(Config.RECONCILER_RETRY_SECONDS_ON_ERROR)
					.build();
		}
	}

	@Override
	public DeleteControl cleanup(OzgKeycloakUser user, Context<OzgKeycloakUser> context) {
		log.info("User reconciler cleanup, keep after delete is set to: " + user.getSpec().isKeepAfterDelete());
		if (user.getSpec().isKeepAfterDelete()) {
			return DeleteControl.defaultDelete();
		}
		return cleanup(user);
	}

	DeleteControl cleanup(OzgKeycloakUser user) {
		var userName = user.getMetadata().getName();
		var namespace = user.getMetadata().getNamespace();
		log.info(String.format("Deleting KeycloakUser %s", userName));
		try {
			service.deleteUser(user.getSpec(), namespace);
			return DeleteControl.defaultDelete();
		} catch (Exception e) {
			log.log(Level.SEVERE, String.format("Could not delete user %s in namespace %s", userName, namespace), e);
			return DeleteControl.defaultDelete();
		}
	}
}
